/*First version March 12, 1997
Simon Potter,


Major revision January 20 1998
This version July 9th 1998

This program calculates marginal likelihoods for 2 
regime univariate setar models with threshold variables
based on the
 (i) previous level of the time series (_a)
(ii) lag difference (_b)
(iii) average recent change (_c).

The program considers the case of  regime specific innovation variance  (hetero) vs  
constant variance across regimes homo (_ah,_bh,_ch).

The program also calculates the marginal likelihood for all lag lengths up to user specified max number (pmax).

Thus, for the nonlinear model pmax*3*2 - 6 marginal likelihoods are calculated.

It also calculates pmax different marginal likelihoods for the linear model.

The output of the program is a table of marginal likelihood values and posterior model probabilities.

The program uses recursive estimation techniques extensively. In particular the add one observation
/delete one observation algorithms. See P.C. Young Recursive Estimation for more information. 


WARNING POSTERIOR MODEL PROBABILITIES CAN BE VERY SENSITIVE TO THE PRIOR.
ONE CAN RUN A SENSITIVITY ANALYSIS EASILY WITH THIS PROGRAM BUT IT IS RECOMMENDED
THAT SOME OF THE HYPERPARAMETERS ARE SET BY CONSIDERING THE SCALE OF THE DATA
&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
USER INPUTS

1. The user needs to specify a data set to be used. 
Two formats are allowed for 

(i) gauss matrix files no requirement to declare the size of the dataset but be careful of
different formats across unix and pc versions

(ii) ascii file, user must declare the size (i.e. number of rows) of the dataset.

In addition there are options to generate artifical data from linear or nonlinear models


2. The user needs to choose pmax and dmax. 

The latter is the max delay considered
for the threshold variable. The marginal likelihood is constructed by averaging across the delays
for each individual model. Note that at present as the program loops around lag lengths the delay is restricted to 
be less than or equal to the current lag length.

3.  The user also needs to choose some `hyperparameters' of the prior distribution of the 
regime coefficients and variances. These are assumed to be indexed by p the lag length but the same across
regimes and for the linear model. This could be changed. What cannot be changed easily is the assumed normal
gamma form.

It is assumed that the priors for the delay and threshold are uniform, hence dmax is a hyperparameter
for the delay and rl, ru are hyperparameters for the threshold described below.

(i)  Regime Coefft  Prior Specification
NOTE ALL PRIOR MEANS ARE SET TO ZERO
a. _hypec  the prior variance of the intercept.
b. _hyped  the prior variance of the first autoregressive coeff't.
c. The variance of the remaining autoregressive cofficients are produced by shrinking down
_hyped, using _hypes within the seqm command. 

(ii) Variance prior specification
a. _v is the degrees of freedom assumed for the gamma distribution. it is set at the default of 3, which means
that the first moment of the prior variance exists for the regime coeff'ts.
b. _v_s is the prior mean for the variance

(iii) Threshold supports are determined by the hyperparameters rl,ru. As presently written the program
will crash if they are chosen in such a way that insufficient data in any regime to construct ols estimates.
rl,ru are set for 15% min in lower regime/upper regime. If the true model generates a lower frequency of
 observations than this
in one of the regimes then although evidence of nonlinearity might be found there will be misspecification as
the correct threshold variable might not be identified.

&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

********************************************************************************************************************/
new;
prtime=hsec;
/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

START DATA INPUT SECTION

@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/



/***************************************************************************************
This part of the program allows you to generate some  some artifical data
********************************************************************************/
/*******************************
First is simple setar model
***************************************************/
/*ttt=400;
u=1*rndn(ttt,1);
y=zeros(ttt,1);
for jj (2,ttt,1);
y[jj,.]=-(1 +.8*y[jj-1,.]).*(y[jj-1,.]>=0)
+(1 -.8*y[jj-1,.]).*(y[jj-1,.]<.0)+u[jj,.];
endfor;*/

/*****************************************
second is ar(1)
***********************************************/
ttt=400;
u=2*rndn(ttt,1);
y=zeros(ttt,1);
for jj (2,ttt,1);
y[jj,.]=(1 +.2*y[jj-1,.])+u[jj,.];
endfor;


/*****************************************************************
Load data from ascii file
*************************************************************************/

/*load _uqnew;
load _uqnew[184,1]=uq.dat;*/


/**********************************
Load data from gauss matrix file, be careful of the format
***********************************************************************/
/*load unewmon;
_uqnew=unewmon[.,2];*/

/*************************************************************
Any required data transformations here, u is the variable name
assumed in the program, 
**********************************************************************/
/*
y=100*ln(_uqnew./(1-_uqnew));*/

u=y[1:rows(y),.];
u=u-meanc(u);

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
END OF DATA INPUT SECTION
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/


/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
START MODEL AND PRIOR CHOICE PART OF THE PROGRAM

&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/

/************************************************************************
Choose maximum lag lengths  
***************************************************************************/

pmax=2;
dmax=2;

pdmax=maxc(pmax|dmax)+1; /*differences will use one more observation*/



/*CONSTRUCTS CONSISTENT SAMPLE FOR ALL THE POSSIBLE LAG LENGTHS*/

t=rows(u);
y=u[pdmax+1:t,.];
xdata=y~ones(t-pdmax,1);
for j (1,pdmax,1);
xdata=xdata~u[pdmax+1-j:t-j,.];
endfor;
t=rows(xdata);

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

PRIOR INPUT SECTION

@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/*************************
Parameters of the prior  for the threshold
*******************************************************/

rl=round(0.15*t);   /*smallest number in lower regime and lowest start point for upper regime*/
 
ru=round(0.85*t);   /*highest end point  for lower regime  and highest start point for upper regime*/

/*************************
Parameters of the prior  for the variance
*******************************************************/
_v=3;    /*degrees of freedom for the variance*/
_v_s=1; /*mean of the variance*/

/*************************
Parameters of the prior  for the regime coefts
*******************************************************/
_hypec=4;   /* variance for the constant*/
_hyped=1;  /* variance for the first ar lag coeff't*/
_hypes=0.8; /*shrinkage factor*/

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

END OF PRIOR INPUT SECTION  AND USER INPUT


@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/


/********************************
Posterior features standard for all models, hence calculate now to save repetition 
**********************************************/
vbar=_v+t;

lgvbar=lnfact(vbar/2-1); 

_c=ln(_v_s)*(_v/2)-lnfact(_v/2-1)+lgvbar; /* constant differs between linear and  nonlinear models, 
using lgvbar to increase its size to avod underflow*/
c=2*_c-lgvbar;


/*STORAGE VECTORS*/
_amlik=zeros(pmax,1);
_ahmlik=zeros(pmax,1);
_bmlik=zeros(pmax,1);
_bhmlik=_amlik;
_cmlik=zeros(pmax,1);
_chmlik=zeros(pmax,1);
mll=_amlik;


for p (1,pmax,1);

_d=(_hypec|_hyped*seqm(1,_hypes,p))/(_v_s/(_v-2)); /*prior for the regression coefficients*/
_hd=diagrv(zeros(2*(p+1),2*(p+1)),_d|_d);
_d=diagrv(zeros(p+1,p+1),_d);

ipq=invpd(_d);
ipq1=ipq;
ipq2=ipq;
dipq=sqrt(det(ipq));
dipq1=dipq;
dipq2=dipq;

_hipq=invpd(_hd);
_hdipq=sqrt(det(_hipq));
d=1; do until d>p;

_abigx=sortc(xdata[.,1:p+2]~xdata[.,d+2],p+2+1);
_bbigx=sortc(xdata[.,1:p+2]~(xdata[.,d+2]-xdata[.,d+3]), p+2+1);

_cbigx=sortc(xdata[.,1:p+2]~((xdata[.,3]-xdata[.,d+3])/d),p+2+1);

/*storage for threshold gaps and intermediate likelihoods*/
_arthres1=0;
_amlikd=0;
_brthres1=0;
_bmlikd=0;
_crthres1=0;
_cmlikd=0;

_ahmlikd=0;
_bhmlikd=0;
_chmlikd=0;

_aml=0;
_bml=0;
_cml=0;
_ahml=0;
_bhml=0;
_chml=0;
_aml123=0;
_ahml123=0;
_bml123=0;
_bhml123=0;
_cml123=0;
_chml123=0;

/*now loop to get all the different marginal likelihoods*/


for i (rl,ru,1) ;
/*storage for intermediate results*/

if i==rl;
_ay1=_abigx[1:rl,1];
_ax1=_abigx[1:rl,2:p+2];
_aq1=moment(_ax1,0);
_abq1=invpd(ipq1+_aq1);
_aqq1=invpd(_aq1);
_ab1=_aqq1*_ax1'_ay1; /*least squares estimator*/
_asee1=(_ay1-_ax1*_ab1)'(_ay1-_ax1*_ab1);

_abb1=_abq1*_ax1'_ay1; /*posterior mean estimator*/
_avbar1=_v+rows(_ay1); /*degrees of freedom for the variance*/


_by1=_bbigx[1:rl,1];
_bx1=_bbigx[1:rl,2:p+2];
_bq1=moment(_bx1,0);
_bbq1=invpd(ipq1+_bq1);
_bqq1=invpd(_bq1);
_bb1=_bqq1*_bx1'_by1; /*least squares estimator*/
_bsee1=(_by1-_bx1*_bb1)'(_by1-_bx1*_bb1);

_bbb1=_bbq1*_bx1'_by1; /*posterior mean estimator*/
_bvbar1=_v+rows(_by1); /*degrees of freedom for the variance*/

if d>1;
_ccc=1;
_cy1=_cbigx[1:rl,1];
_cx1=_cbigx[1:rl,2:p+2];
_cq1=moment(_cx1,0);
_cbq1=invpd(ipq1+_cq1);
_cqq1=invpd(_cq1);
_cb1=_cqq1*_cx1'_cy1; /*least squares estimator*/
_csee1=(_cy1-_cx1*_cb1)'(_cy1-_cx1*_cb1);

_cbb1=_cbq1*_cx1'_cy1; /*posterior mean estimator*/
_cvbar1=_v+rows(_cy1); /*degrees of freedom for the variance*/
else;
_ccc=0;
endif;

else;
/*use the add one in estimator to recursively update the required quanitities*/
_ay1=_ay1|_abigx[i,1];
_ax1=_ax1|_abigx[i,2:p+2];
_avbar1=_avbar1+1;
x=_abigx[i,2:p+2];
xx=moment(x,0);
fq=x*_aqq1*x'+1;
fqb=x*_abq1*x'+1;
_ab1=_ab1+_aqq1*x'*(_ay1[i,1]-x*_ab1)/fq;
_aq1=_aq1+xx;
_aqq1=_aqq1-_aqq1*xx*_aqq1/(fq);
_asee1=(_ay1-_ax1*_ab1)'(_ay1-_ax1*_ab1);
_abb1=_abb1+_abq1*x'*(_ay1[i,1]-x*_abb1)/fqb;
_abq1=_abq1-_abq1*xx*_abq1/(fqb);




/*use the add one in estimator to recursively update the required quanitities*/
_by1=_by1|_bbigx[i,1];
_bx1=_bx1|_bbigx[i,2:p+2];
_bvbar1=_bvbar1+1;
x=_bbigx[i,2:p+2];
xx=moment(x,0);
fq=x*_bqq1*x'+1;
fqb=x*_bbq1*x'+1;
_bb1=_bb1+_bqq1*x'*(_by1[i,1]-x*_bb1)/fq;
_bq1=_bq1+xx;
_bqq1=_bqq1-_bqq1*xx*_bqq1/(fq);
_bsee1=(_by1-_bx1*_bb1)'(_by1-_bx1*_bb1);
_bbb1=_bbb1+_bbq1*x'*(_by1[i,1]-x*_bbb1)/fqb;
_bbq1=_bbq1-_bbq1*xx*_bbq1/(fqb);

if d>1;
_cy1=_cy1|_cbigx[i,1];
_cx1=_cx1|_cbigx[i,2:p+2];
_cvbar1=_cvbar1+1;
x=_cbigx[i,2:p+2];
xx=moment(x,0);
fq=x*_cqq1*x'+1;
fqb=x*_cbq1*x'+1;
_cb1=_cb1+_cqq1*x'*(_cy1[i,1]-x*_cb1)/fq;
_cq1=_cq1+xx;
_cqq1=_cqq1-_cqq1*xx*_cqq1/(fq);
_csee1=(_cy1-_cx1*_cb1)'(_cy1-_cx1*_cb1);
_cbb1=_cbb1+_cbq1*x'*(_cy1[i,1]-x*_cbb1)/fqb;
_cbq1=_cbq1-_cbq1*xx*_cbq1/(fqb);
endif;

endif;
_ar1=_abigx[i+1,cols(_abigx)]-_abigx[i,cols(_abigx)];  /*for the
 overall averagi
ng*/
_avsbar1=_v_s+_asee1+(_abb1-_ab1)'_aq1*(_abb1-_ab1)+(_abb1'ipq1*_abb1);
_ad1=sqrt(det(_abq1));
_aml1=lnfact(_avbar1/2-1)+ln(_ad1)+ln(dipq1)+ln(_avsbar1)*(-_avbar1/2);


_br1=_bbigx[i+1,cols(_bbigx)]-_bbigx[i,cols(_bbigx)];
_bvsbar1=_v_s+_bsee1+(_bbb1-_bb1)'_bq1*(_bbb1-_bb1)+(_bbb1'ipq1*_bbb1);
_bd1=sqrt(det(_bbq1));
_bml1=lnfact(_bvbar1/2-1)+ln(_bd1)+ln(dipq1)+ln(_bvsbar1)*(-_bvbar1/2);

if d>1;
_cr1=_cbigx[i+1,cols(_cbigx)]-_cbigx[i,cols(_cbigx)];
/*for the overall averaging*/
_cvsbar1=_v_s+_csee1+(_cbb1-_cb1)'_cq1*(_cbb1-_cb1)+(_cbb1'ipq1*_cbb1);
_cd1=sqrt(det(_cbq1));
_cml1=lnfact(_cvbar1/2-1)+ln(_cd1)+ln(dipq1)+ln(_cvsbar1)*(-_cvbar1/2);
endif;
if i==rl;
_ay2=_abigx[1+i:t,1];  /*middle regime*/
_ax2=_abigx[1+i:t,2:p+2];





_aq2=moment(_ax2,0);
_abq2=invpd(ipq2+_aq2);
_aqq2=invpd(_aq2);
_ab2=_aqq2*_ax2'_ay2; /*least squares estimator*/
_asee2=(_ay2-_ax2*_ab2)'(_ay2-_ax2*_ab2);

_abb2=_abq2*_ax2'_ay2; /*posterior mean estimator*/
_avbar2=_v+rows(_ay2); /*degrees of freedom for the variance*/






_by2=_bbigx[1+i:t,1];  /*upper regime*/
_bx2=_bbigx[1+i:t,2:p+2];





_bq2=moment(_bx2,0);
_bbq2=invpd(ipq2+_bq2);
_bqq2=invpd(_bq2);
_bb2=_bqq2*_bx2'_by2; /*least squares estimator*/
_bsee2=(_by2-_bx2*_bb2)'(_by2-_bx2*_bb2);

_bbb2=_bbq2*_bx2'_by2; /*posterior mean estimator*/
_bvbar2=_v+rows(_by2); /*degrees of freedom for the variance*/


if d>1;
_cccc=1;

_cy2=_cbigx[1+i:t,1];  /*middle regime*/
_cx2=_cbigx[1+i:t,2:p+2];




_cq2=moment(_cx2,0);
_cbq2=invpd(ipq2+_cq2);
_cqq2=invpd(_cq2);
_cb2=_cqq2*_cx2'_cy2; /*least squares estimator*/
_csee2=(_cy2-_cx2*_cb2)'(_cy2-_cx2*_cb2);

_cbb2=_cbq2*_cx2'_cy2; /*posterior mean estimator*/
_cvbar2=_v+rows(_cy2); /*degrees of freedom for the variance*/

endif;



else;
/*use the take away one in estimator to recursively update the
 required quantities for the upper regime and
 the add one estimator in for the
middle regime*/

_ay2=_abigx[i+1:t,1];
_ax2=_abigx[i+1:t,2:p+2];


_avbar2=_avbar2-1;

/*take away one estimator for upper regime*/
x=_abigx[i,2:p+2];
y=_abigx[i,1];
xx=moment(x,0);
fq=x*_aqq2*x'-1;
fqb=x*_abq2*x'-1;
_ab2=_ab2+_aqq2*x'*(y-x*_ab2)/fq;
_asee2=(_ay2-_ax2*_ab2)'(_ay2-_ax2*_ab2);
_abb2=_abb2+_abq2*x'*(y-x*_abb2)/fqb;
_aq2=_aq2-xx;
_aqq2=_aqq2-_aqq2*xx*_aqq2/(fq);
_abq2=_abq2-_abq2*xx*_abq2/(fqb);








_by2=_bbigx[i+1:t,1];
_bx2=_bbigx[i+1:t,2:p+2];

_bvbar2=_bvbar2-1;

/*take away one estimator for upper regime*/
x=_bbigx[i,2:p+2];
y=_bbigx[i,1];
xx=moment(x,0);
fq=x*_bqq2*x'-1;
fqb=x*_bbq2*x'-1;
_bb2=_bb2+_bqq2*x'*(y-x*_bb2)/fq;
_bsee2=(_by2-_bx2*_bb2)'(_by2-_bx2*_bb2);
_bbb2=_bbb2+_bbq2*x'*(y-x*_bbb2)/fqb;
_bq2=_bq2-xx;
_bqq2=_bqq2-_bqq2*xx*_bqq2/(fq);
_bbq2=_bbq2-_bbq2*xx*_bbq2/(fqb);
if d>1  ;

_cy2=_cbigx[i+1:t,1];
_cx2=_cbigx[i+1:t,2:p+2];


_cvbar2=_cvbar2-1;

/*take away estimator for upper regime*/
x=_cbigx[i,2:p+2];
y=_cbigx[i,1];
xx=moment(x,0);
fq=x*_cqq2*x'-1;
fqb=x*_cbq2*x'-1;
_cb2=_cb2+_cqq2*x'*(y-x*_cb2)/fq;
_csee2=(_cy2-_cx2*_cb2)'(_cy2-_cx2*_cb2);
_cbb2=_cbb2+_cbq2*x'*(y-x*_cbb2)/fqb;
_cq2=_cq2-xx;
_cqq2=_cqq2-_cqq2*xx*_cqq2/(fq);
_cbq2=_cbq2-_cbq2*xx*_cbq2/(fqb);



endif;



endif;

_avsbar2=_v_s+_asee2+(_abb2-_ab2)'_aq2*(_abb2-_ab2)+(_abb2'ipq2*_abb2);
_ad2=sqrt(det(_abq2));
_aml2=lnfact(_avbar2/2-1)+ln(_ad2)+ln(dipq2)+ln(_avsbar2)*(-_avbar2/2);

_aml123=_aml123|(_aml1+_aml2);




_bvsbar2=_v_s+_bsee2+(_bbb2-_bb2)'_bq2*(_bbb2-_bb2)+(_bbb2'ipq2*_bbb2);
_bd2=sqrt(det(_bbq2));
_bml2=lnfact(_bvbar2/2-1)+ln(_bd2)+ln(dipq2)+ln(_bvsbar2)*(-_bvbar2/2);

_bml123=_bml123|(_bml1+_bml2);


if d>1;


_cvsbar2=_v_s+_csee2+(_cbb2-_cb2)'_cq2*(_cbb2-_cb2)+(_cbb2'ipq2*_cbb2);
_cd2=sqrt(det(_cbq2));
_cml2=lnfact(_cvbar2/2-1)+ln(_cd2)+ln(dipq2)+ln(_cvsbar2)*(-_cvbar2/2);

_cml123=_cml123|(_cml1+_cml2);

endif;


/*now calculate for homo models can reduce number of calculations
drastically since mostly same as the hetero model*/


_ahvsbar=-_v_s+_avsbar1+_avsbar2;

_ahml123=
_ahml123|exp(_c+lgvbar+ln(_ad1*_ad2*_hdipq)-(vbar/2)*ln(_ahvsbar));



_bhvsbar=-_v_s+_bvsbar1+_bvsbar2;


_bhml123=
_bhml123|exp(_c+lgvbar+ln(_bd1*_bd2*_hdipq)-(vbar/2)*ln(_bhvsbar));



if d>1;

_chvsbar=-_v_s+_cvsbar1+_cvsbar2;


_chml123=
_chml123|exp(_c+lgvbar+ln(_cd1*_cd2*_hdipq)-(vbar/2)*ln(_chvsbar));


endif;


alex:

_arthres1=_arthres1|_ar1;

/*average out over the threshold*/



_brthres1=_brthres1|_br1;

/*average out over the thresholds, c was constant so introduced now*/

if d>1;

_crthres1=_crthres1|_cr1;


endif;
endfor;

_aml123[2:rows(_aml123),.]=exp(_aml123[2:rows(_aml123),.]+c);
_bml123[2:rows(_bml123),.]=exp(_bml123[2:rows(_bml123),.]+c);
if d>1;
_cml123[2:rows(_cml123),.]=exp(_cml123[2:rows(_cml123),.]+c);
endif;
if sumc(_arthres1)>0;
_arthres1=_arthres1/sumc(_arthres1);
_amlikd=_amlikd|(_aml123'_arthres1);
_ahmlikd=_ahmlikd|(_ahml123'_arthres1);
endif;
if sumc(_brthres1)>0;
_brthres1=_brthres1/sumc(_brthres1);
_bmlikd=_bmlikd|(_bml123'_brthres1);
_bhmlikd=_bhmlikd|(_bhml123'_brthres1);
endif;
if d>1 and sumc(_crthres1)>0;
_crthres1=_crthres1/sumc(_crthres1);
_cmlikd=_cmlikd|(_cml123'_crthres1);
_chmlikd=_chmlikd|(_chml123'_crthres1);
endif;

d=d+1;endo;

_amlik[p,.]=sumc(_amlikd)/(d-1);
_ahmlik[p,.]=sumc(_ahmlikd)/(d-1);

_bmlik[p,.]=sumc(_bmlikd)/(d-1);
_bhmlik[p,.]=sumc(_bhmlikd)/(d-1);
if p>1 and d>2;
_cmlik[p,.]=sumc(_cmlikd)/(d-2);
_chmlik[p,.]=sumc(_chmlikd)/(d-2);
endif;

/*calculate marginal likelihood for AR(p)*/


y=xdata[.,1];
x=xdata[.,2:p+2];
q=moment(x,0);
bq=invpd(ipq+q);
qq=invpd(q);
b=qq*x'y; /*least squares estimator*/
see=(y-x*b)'(y-x*b);

bb=bq*x'y; /*posterior mean estimator*/
vsbar=_v_s+see+(bb-b)'q*(bb-b)+bb'ipq*bb;

mll[p,.]=exp(_c+
lgvbar+ln(sqrt(det(bq))*dipq)-(vbar/2)*ln(vsbar));


endfor;

/*now do the calculations for the posterior model probabilities*/
totml=_amlik~_ahmlik~_bmlik~_bhmlik~_cmlik~_chmlik~mll;
postm=totml/sumc(sumc(totml));

"Posterior Model Probabs, a, ah, b, bh, c , ch, Lar and rows are p  ";
postm;
x=sumc(sumc(postm[.,1:6]));
"nonlinear vs linear  ";
x~sumc(postm[.,7]);


"time to run in minutes " (hsec-prtime)/6000;
save postm;


