/*proc to calculate GI from posterior of two regime tar model
regime based on lagged differences
*/
proc gi2tard(b1,b2,r1,d,s1,s2,_x,k,n,v1,v2,nn);

/*********************************************
b1,b2 are the regime ar coefficients, with intercept in first row,
eg b1 is p+1 X k, K represents number of draws from the posterior, the
rows are the coefficients for individual lags
r1,d are scalars and drawn in the main program,
s1,s2 are all 1 X K and are the regime standard deviations,
1 is the lower regime, 2 is the upper regime
_x is the 1 X (p+1) set of initial conditions
n is number of monte carlo draws for the gi,
v1 is the postive shock,
v2 is the negative shock
nn is the horizon for the impulse response function
****************************************************************/

local y,x,y1,x1,y2,x2,y0,x0,u,i,j,p,_xhat,_xhat1,_xhat2,
e1,e2,e3,e10,e20,e30,e11,e21,e31,e12,e22,e32,gi;

p=rows(b1)-1;

/*find gi_0 and set up the matrices*/


e1=((_x[d,.]-_x[d+1,.])<=r1);

e2=(1-e1);
_xhat=(b1[1,.].*e1+b2[1,.].*e2);
for j (1,p,1);
_xhat=_xhat+_x[j,.]*(e1.*b1[j+1,.]+e2.*b2[j+1,.]);
endfor;

/*now set up baseline simulation*/
u=rndn(n,1);
x=reshape(_x[1,.],n,k);
for j (2,p+1,1);
x=x~reshape(_x[j,.],n,k);
endfor;

y=(b1[1,.].*e1+b2[1,.].*e2)+(s1*e1+s2*e2).*u;
for j (1,p,1);
y=y+x[.,(j-1)*k+1:j*k].*(e1.*b1[j+1,.]+e2.*b2[j+1,.]);
endfor;
/*make sure first set of draws is centered
correctly*/
y=y-meanc(y)'+_xhat;

_xhat1=_xhat+(s1*e1+s2*e2)*v1;
_xhat2=_xhat+(s1*e1+s2*e2)*v2;

x1=x;
x2=x;
x0=x;
if p>1;
x[.,k+1:k*(p+1)]=x[.,1:(p)*k];
x1[.,k+1:k*(p+1)]=x1[.,1:(p)*k];
x2[.,k+1:k*(p+1)]=x2[.,1:(p)*k];
x0[.,k+1:k*(p+1)]=x0[.,1:(p)*k];
endif;
x[.,1:k]=y;
x0[.,1:k]=ones(n,1).*_xhat;
x1[.,1:k]=ones(n,1).*_xhat1;
x2[.,1:k]=ones(n,1).*_xhat2;
gi=zeros(nn+1,3);
y=meanc(meanc(y));
gi[1,1]=meanc(_xhat1')-y;
gi[1,2]=meanc(_xhat2')-y;



for i (1, nn,1);





u=rndn(n,1);
e1=((x[.,(d-1)*k+1:d*k]-x[.,d*k+1:(d+1)*k]).<=r1);
e2=(1-e1);
y=(b1[1,.].*e1+b2[1,.].*e2)+(s1.*e1+s2.*e2).*u;
e11=((x1[.,(d-1)*k+1:d*k]-x1[.,d*k+1:(d+1)*k]).<=r1);
e21=(1-e11);
y1=(b1[1,.].*e11+b2[1,.].*e21)+(s1.*e11+s2.*e21).*u;
e12=((x2[.,(d-1)*k+1:d*k]-x2[.,d*k+1:(d+1)*k]).<=r1);
e22=(1-e12);
y2=(b1[1,.].*e12+b2[1,.].*e22)+(s1.*e12+s2.*e22).*u;
e10=((x0[.,(d-1)*k+1:d*k]-x0[.,d*k+1:(d+1)*k]).<=r1);
e20=(1-e10);
y0=(b1[1,.].*e10+b2[1,.].*e20)+(s1.*e10+s2.*e20).*(u);

for j (1,p,1);
y=y+x[.,(j-1)*k+1:j*k].*(e1.*b1[j+1,.]+e2.*b2[j+1,.]);
y1=y1+x1[.,(j-1)*k+1:j*k].*(e11.*b1[j+1,.]+e21.*b2[j+1,.]);
y2=y2+x2[.,(j-1)*k+1:j*k].*(e12.*b1[j+1,.]+e22.*b2[j+1,.]);
 y0=y0+x0[.,(j-1)*k+1:j*k].*(e10.*b1[j+1,.]+e20.*b2[j+1,.]);
endfor;
if p>1;
x[.,k+1:k*p]=x[.,1:(p-1)*k];
x1[.,k+1:k*p]=x1[.,1:(p-1)*k];
x2[.,k+1:k*p]=x2[.,1:(p-1)*k];
x0[.,k+1:k*p]=x0[.,1:(p-1)*k];
endif;
x[.,1:k]=y;
x0[.,1:k]=y0;
x1[.,1:k]=y1;
x2[.,1:k]=y2;
/*need to apply the reverse (logistic) transformation for unemployment paper*/
y=meanc(meanc(y));
gi[i+1,1]=meanc(meanc(y1))-y;
gi[i+1,2]=meanc(meanc(y2))-y;
gi[i+1,3]=meanc(meanc(y0))-y;
endfor;

retp(gi[.,1:2]);

endp;
